{-# LANGUAGE OverloadedStrings #-}
module Main where

import Data.ByteString (ByteString, writeFile, hPut, length)
import Network.Download (openURI)
import qualified Data.Text.Lazy as T (Text, unpack, pack)
import qualified Data.Text.Format as TF (format, Only (Only))
import Options.Applicative.Common
import Options.Applicative.Builder
import Options.Applicative.Extra
import Control.Monad
import Prelude hiding (writeFile, length)
import System.Directory
import Control.Concurrent (threadDelay)

getStockDataDir :: IO FilePath
getStockDataDir = do
  homeDir <- getHomeDirectory
  return $ homeDir ++ "/.stockdata"

tickerToURL :: T.Text -> T.Text
tickerToURL t =
  TF.format "http://real-chart.finance.yahoo.com/table.csv?s={}&d=0&e=27&f=2099&g=d&a=0&b=3&c=1950&ignore=.csv" (TF.Only t)

processStockData :: FilePath -> T.Text -> IO ()
processStockData fp t = do
  f <- openURI $ T.unpack (tickerToURL t)
  print f
  let result = case f of
            (Right v) -> (t, Just v)
            (Left e) -> (t, Nothing)
  writeStockData fp result
  print $ maybe 0 length (snd result)

data Arguments = Arguments
  { tickerFile :: String }

arguments :: Parser Arguments
arguments = Arguments
  <$> strOption
      ( long "file"
     <> metavar "TICKERFILE"
     <> help "File containing a ticker symbol per line" )

fileName :: IO String
fileName = fmap tickerFile (execParser $ info arguments fullDesc)

writeStockData :: FilePath -> (T.Text, Maybe ByteString) -> IO ()
writeStockData p (t, Just v) = do
  writeFile (T.unpack $ TF.format "{}/{}.csv" (p, t)) v
  print (length v)
writeStockData _ (t, Nothing) = putStrLn $ T.unpack t ++ " was not succesfully fetched."

main :: IO ()
main = do
  tickerData <- fileName >>= readFile
  let tickers = map T.pack (lines tickerData)
  stockDataDir <- getStockDataDir
  createDirectoryIfMissing False stockDataDir
  mapM_ (processStockData stockDataDir) tickers
